@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-#    Copyright (C) 2012-2016 Christian T. Jacobs.
+#    Copyright (C) 2012-2017 Christian Thomas Jacobs.
 
 #    This file is part of PyQSO.
 
@@ -19,10 +19,12 @@
 
 import re
 import logging
-import unittest
 from datetime import datetime
 import calendar
-import configparser
+try:
+    import configparser
+except ImportError:
+    import ConfigParser as configparser
 from os.path import expanduser
 
 # ADIF field names and their associated data types available in PyQSO.
@@ -85,6 +87,7 @@ AVAILABLE_FIELD_NAMES_FRIENDLY = {"CALL": "Callsign",
 # M: Multi-line string
 # G: Multi-line international string
 # L: Location
+# E: Enumerated
 DATA_TYPES = ["A", "B", "N", "S", "I", "D", "T", "M", "G", "L", "E"]
 
 # All the valid modes listed in the ADIF specification. This is a dictionary with the key-value pairs holding the MODE and SUBMODE(s) respectively.
@@ -193,41 +196,35 @@ class ADIF:
 
     def __init__(self):
         """ Initialise class for I/O of files using the Amateur Data Interchange Format (ADIF). """
-        logging.debug("New ADIF instance created!")
+        return
 
     def read(self, path):
         """ Read an ADIF file and parse it.
 
         :arg str path: The path to the ADIF file to read.
-        :returns: A list of dictionaries (one dictionary per QSO), with each dictionary containing field-value pairs, e.g. {FREQ:145.500, BAND:2M, MODE:FM}.
+        :returns: A list of dictionaries (one dictionary per QSO), with each dictionary containing field-value pairs, e.g. {FREQ:145.500, BAND:2M, MODE:FM}. If the file cannot be read, the method returns None.
         :rtype: list
-        :raises IOError: if the ADIF file does not exist or cannot be read (e.g. due to lack of read permissions).
+        :raises IOError: If the ADIF file does not exist or cannot be read (e.g. due to lack of read permissions).
         """
         logging.debug("Reading in ADIF file with path: %s..." % path)
 
         text = ""
-        try:
-            f = open(path, mode='r', errors="replace")
+        with open(path, mode="r", errors="replace") as f:
             text = f.read()
-            f.close()  # Close the file, otherwise "bad things" might happen!
-        except IOError as e:
-            logging.error("I/O error %d: %s" % (e.errno, e.strerror))
-        except Exception as e:
-            logging.error("An error occurred when reading the ADIF file.")
-            logging.exception(e)
 
-        records = self._parse_adi(text)
+        records = self.parse_adi(text)
 
         if(records == []):
             logging.warning("No records found in the file. Empty file or wrong file type?")
 
+        logging.info("Read %d QSOs from %s in ADIF format." % (len(records), path))
         return records
 
-    def _parse_adi(self, text):
+    def parse_adi(self, text):
         """ Parse some raw text (defined in the 'text' argument) for ADIF field data.
 
         :arg str text: The raw text from the ADIF file to parse.
-        :returns: A list of dictionaries (one dictionary per QSO). Each dictionary contains the field-value pairs, e.g. {FREQ:145.500, BAND:2M, MODE:FM}.
+        :returns: A list of dictionaries (one dictionary per QSO). Each dictionary contains the field-value pairs, e.g. {"FREQ": "145.500", "BAND": "2M", "MODE": "FM"}.
         :rtype: list
         """
 
@@ -237,37 +234,34 @@ class ADIF:
 
         # ADIF-related configuration options
         config = configparser.ConfigParser()
-        have_config = (config.read(expanduser('~/.config/pyqso/preferences.ini')) != [])
-        (section, option) = ("adif", "merge_comment")
-        if(have_config and config.has_option(section, option) and config.get(section, option) == "True"):
+        have_config = (config.read(expanduser("~/.config/pyqso/preferences.ini")) != [])
+        (section, option) = ("import_export", "merge_comment")
+        if(have_config and config.has_option(section, option) and config.getboolean(section, option)):
             merge_comment = True
         else:
             merge_comment = False
 
         # Separate the text at the <eor> or <eoh> markers.
-        tokens = re.split('(<eor>|<eoh>)', text, flags=re.IGNORECASE)
+        tokens = re.split("(<eor>|<eoh>)", text, flags=re.IGNORECASE)
         tokens.pop()  # Anything after the final <eor> marker should be ignored.
 
         # The header might tell us the number of records, but let's not assume
         # this and simply ignore it instead (if it exists).
-        if(re.search('<eoh>', text, flags=re.IGNORECASE) is not None):
+        if(re.search("<eoh>", text, flags=re.IGNORECASE) is not None):
             # There is a header present, so let's ignore everything
-            # up to and including the <eoh> marker. Note that
-            # re.search has been used here to handle any case sensitivity.
-            # Previously we were checking for <eoh>. <EOH> is also valid
-            # but wasn't being detected before.
+            # up to and including the <eoh> marker.
             while len(tokens) > 0:
                 t = tokens.pop(0)
-                if(re.match('<eoh>', t, flags=re.IGNORECASE) is not None):
+                if(re.match("<eoh>", t, flags=re.IGNORECASE) is not None):
                     break
 
         n_eor = 0
         n_record = 0
         records = []
-        pattern = re.compile('<(.*?):(\d*).*?>([^<\t\n\r\f\v]+)')
+        pattern = re.compile("<(.*?):(\d*).*?>([^<]+)")
 
         for t in tokens:
-            if(re.match('<eor>', t, flags=re.IGNORECASE) is not None):
+            if(re.match("<eor>", t, flags=re.IGNORECASE) is not None):
                 n_eor += 1
                 continue
             else:
@@ -284,6 +278,7 @@ class ADIF:
                     # This will help us later when comparing the field names
                     # against the available field names in the ADIF specification.
                     field_name = fd[0].upper()
+                    # Only read in the number of characters specified by the data length.
                     field_data = fd[2][:int(fd[1])]
 
                     # Combo boxes are used later on and these are case sensitive,
@@ -306,7 +301,7 @@ class ADIF:
                 if(merge_comment):
                     if("NOTES" in list(fields_and_data_dictionary.keys()) and comment):
                         logging.debug("Merging COMMENT field with NOTES field...")
-                        fields_and_data_dictionary["NOTES"] += "\\n" + comment
+                        fields_and_data_dictionary["NOTES"] += "\n" + comment
                         logging.debug("Merged fields.")
                     elif(comment):
                         # Create the NOTES entry, but only store the contents of the COMMENT field.
@@ -328,12 +323,12 @@ class ADIF:
         :arg list records: The list of QSO records to write.
         :arg str path: The desired path of the ADIF file to write to.
         :returns: None
-        :raises IOError: if the ADIF file cannot be written (e.g. due to lack of write permissions).
+        :raises IOError: If the ADIF file cannot be written (e.g. due to lack of write permissions).
         """
 
         logging.debug("Writing records to an ADIF file...")
-        try:
-            f = open(path, mode='w', errors="replace")  # Open file for writing
+
+        with open(path, mode="w", errors="replace") as f:  # Open file for writing
 
             # First write a header containing program version, number of records, etc.
             dt = datetime.now()
@@ -342,10 +337,10 @@ class ADIF:
 
 <adif_ver:%d>%s
 <programid:5>PyQSO
-<programversion:3>0.3
+<programversion:5>1.0.0
 <eoh>\n""" % (dt, len(records), len(str(ADIF_VERSION)), ADIF_VERSION))
 
-            # Then write each log to the file.
+            # Then write each record to the file.
             for r in records:
                 for field_name in AVAILABLE_FIELD_NAMES_ORDERED:
                     if(not(field_name.lower() in list(r.keys()) or field_name.upper() in list(r.keys()))):
@@ -360,11 +355,7 @@ class ADIF:
             logging.debug("Finished writing records to the ADIF file.")
             f.close()
 
-        except IOError as e:
-            logging.error("I/O error %d: %s" % (e.errno, e.strerror))
-        except Exception as e:  # All other exceptions.
-            logging.error("An error occurred when writing the ADIF file.")
-            logging.exception(e)
+            logging.info("Wrote %d QSOs to %s in ADIF format." % (len(records), path))
 
         return
 
@@ -380,9 +371,9 @@ class ADIF:
 
         logging.debug("Validating the following data in field '%s': %s" % (field_name, data))
 
-        # Allow an empty string, in case the user doesn't want
+        # Allow an empty string or None, in case the user doesn't want
         # to fill in this field.
-        if(data == ""):
+        if(not data):
             return True
 
         if(data_type == "N"):
@@ -520,6 +511,9 @@ class ADIF:
             # Enumeration, AwardList.
             if(field_name == "MODE"):
                 return (data in list(MODES.keys()))
+            elif(field_name == "SUBMODE"):
+                submodes = [submode for mode in list(MODES.keys()) for submode in MODES[mode]]
+                return (data in submodes)
             elif(field_name == "BAND"):
                 return (data in BANDS)
             else:
@@ -527,195 +521,3 @@ class ADIF:
 
         else:
             return True
-
-
-class TestADIF(unittest.TestCase):
-
-    """ The unit tests for the ADIF module. """
-
-    def setUp(self):
-        """ Set up the ADIF object needed for the unit tests. """
-        self.adif = ADIF()
-
-    def test_adif_read(self):
-        """ Check that a single ADIF record can be read and parsed correctly. """
-        f = open("ADIF.test_read.adi", 'w')
-        f.write("""Some test ADI data.<eoh>
-
-<call:4>TEST<band:3>40m<mode:2>CW
-<qso_date:8:d>20130322<time_on:4>1955<eor>""")
-        f.close()
-
-        records = self.adif.read("ADIF.test_read.adi")
-        expected_records = [{'TIME_ON': '1955', 'BAND': '40m', 'CALL': 'TEST', 'MODE': 'CW', 'QSO_DATE': '20130322'}]
-        print("Imported records: ", records)
-        print("Expected records: ", expected_records)
-        assert(len(records) == 1)
-        assert(len(list(records[0].keys())) == len(list(expected_records[0].keys())))
-        assert(records == expected_records)
-
-    def test_adif_read_multiple(self):
-        """ Check that multiple ADIF records can be read and parsed correctly. """
-        f = open("ADIF.test_read_multiple.adi", 'w')
-        f.write("""Some test ADI data.<eoh>
-
-<call:4>TEST<band:3>40m<mode:2>CW
-<qso_date:8:d>20130322<time_on:4>1955<eor>
-
-<call:8>TEST2ABC<band:3>20m<mode:3>SSB
-<qso_date:8>20150227<time_on:4>0820<eor>
-
-<call:5>HELLO<band:2>2m<mode:2>FM<qso_date:8:d>20150227<time_on:4>0832<eor>""")
-        f.close()
-
-        records = self.adif.read("ADIF.test_read_multiple.adi")
-        expected_records = [{'TIME_ON': '1955', 'BAND': '40m', 'CALL': 'TEST', 'MODE': 'CW', 'QSO_DATE': '20130322'}, {'TIME_ON': '0820', 'BAND': '20m', 'CALL': 'TEST2ABC', 'MODE': 'SSB', 'QSO_DATE': '20150227'}, {'TIME_ON': '0832', 'BAND': '2m', 'CALL': 'HELLO', 'MODE': 'FM', 'QSO_DATE': '20150227'}]
-        print("Imported records: ", records)
-        print("Expected records: ", expected_records)
-        assert(len(records) == 3)
-        for i in range(len(expected_records)):
-            assert(len(list(records[i].keys())) == len(list(expected_records[i].keys())))
-        assert(records == expected_records)
-
-    def test_adif_read_alphabet(self):
-        """ Check that none of the letters of the alphabet are ignored during parsing. """
-        f = open("ADIF.test_read_alphabet.adi", 'w')
-        f.write("""Some test ADI data.<eoh>
-<call:64>ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ<eor>""")
-        f.close()
-
-        records = self.adif.read("ADIF.test_read_alphabet.adi")
-        expected_records = [{'CALL': 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'}]
-        print("Imported records: ", records)
-        print("Expected records: ", expected_records)
-        assert(len(records) == 1)
-        assert(len(list(records[0].keys())) == len(list(expected_records[0].keys())))
-        assert(records == expected_records)
-
-    def test_adif_read_capitalisation(self):
-        """ Check that the CALL field is capitalised correctly. """
-        f = open("ADIF.test_read_capitalisation.adi", 'w')
-        f.write("""Some test ADI data.<eoh>
-<call:4>test<eor>""")
-        f.close()
-
-        records = self.adif.read("ADIF.test_read_capitalisation.adi")
-        expected_records = [{'CALL': 'TEST'}]
-        print("Imported records: ", records)
-        print("Expected records: ", expected_records)
-        assert(len(records) == 1)
-        assert(len(list(records[0].keys())) == len(list(expected_records[0].keys())))
-        assert(records == expected_records)
-
-    def test_adif_read_header_only(self):
-        """ Check that no records are read in if the ADIF file only contains header information. """
-        f = open("ADIF.test_read_header_only.adi", 'w')
-        f.write("""Some test ADI data.<eoh>""")
-        f.close()
-
-        records = self.adif.read("ADIF.test_read_header_only.adi")
-        expected_records = []
-        print("Imported records: ", records)
-        print("Expected records: ", expected_records)
-        assert(len(records) == 0)
-        assert(records == expected_records)
-
-    def test_adif_read_no_header(self):
-        """ Check that an ADIF file can be parsed with no header information. """
-        f = open("ADIF.test_read_no_header.adi", 'w')
-        f.write("""<call:4>TEST<band:3>40m<mode:2>CW<qso_date:8:d>20130322<time_on:4>1955<eor>""")
-        f.close()
-
-        records = self.adif.read("ADIF.test_read_no_header.adi")
-        expected_records = [{'TIME_ON': '1955', 'BAND': '40m', 'CALL': 'TEST', 'MODE': 'CW', 'QSO_DATE': '20130322'}]
-        print("Imported records: ", records)
-        print("Expected records: ", expected_records)
-        assert(len(records) == 1)
-        assert(len(list(records[0].keys())) == len(list(expected_records[0].keys())))
-        assert(records == expected_records)
-
-    def test_adif_write(self):
-        """ Check that records can be written to an ADIF file correctly. """
-        records = [{"CALL": "TEST123", "QSO_DATE": "20120402", "TIME_ON": "1234", "FREQ": "145.500", "BAND": "2m", "MODE": "FM", "RST_SENT": "59", "RST_RCVD": "59"},
-                   {"CALL": "TEST123", "QSO_DATE": "20130312", "TIME_ON": "0101", "FREQ": "145.750", "BAND": "2m", "MODE": "FM"}]
-        self.adif.write(records, "ADIF.test_write.adi")
-
-        f = open("ADIF.test_write.adi", 'r')
-        text = f.read()
-        print("File 'ADIF.test_write.adi' contains the following text:", text)
-        assert("""
-<adif_ver:5>3.0.4
-<programid:5>PyQSO
-<programversion:3>0.3
-<eoh>
-<call:7>TEST123
-<qso_date:8>20120402
-<time_on:4>1234
-<freq:7>145.500
-<band:2>2m
-<mode:2>FM
-<rst_sent:2>59
-<rst_rcvd:2>59
-<eor>
-<call:7>TEST123
-<qso_date:8>20130312
-<time_on:4>0101
-<freq:7>145.750
-<band:2>2m
-<mode:2>FM
-<eor>
-""" in text)  # Ignore the header line here, since it contains the date and time the ADIF file was written, which will change each time 'make unittest' is run.
-        f.close()
-
-    def test_adif_write_sqlite3_Row(self):
-        """ Check that records can be written to an ADIF file from a test database file. """
-        import sqlite3
-        import os.path
-        self.connection = sqlite3.connect(os.path.dirname(os.path.realpath(__file__))+"/unittest_resources/test.db")
-        self.connection.row_factory = sqlite3.Row
-
-        c = self.connection.cursor()
-        c.execute("SELECT * FROM test")
-        records = c.fetchall()
-        print(records)
-
-        self.adif.write(records, "ADIF.test_write_sqlite3_Row.adi")
-
-        f = open("ADIF.test_write_sqlite3_Row.adi", 'r')
-        text = f.read()
-        print("File 'ADIF.test_write_sqlite3_Row.adi' contains the following text:", text)
-        assert("""
-<adif_ver:5>3.0.4
-<programid:5>PyQSO
-<programversion:3>0.3
-<eoh>
-<call:7>TEST123
-<qso_date:8>20120402
-<time_on:4>1234
-<freq:7>145.500
-<band:2>2m
-<mode:2>FM
-<rst_sent:2>59
-<rst_rcvd:2>59
-<eor>
-<call:7>TEST456
-<qso_date:8>20130312
-<time_on:4>0101
-<freq:7>145.750
-<band:2>2m
-<mode:2>FM
-<eor>
-""" in text)  # Ignore the header line here, since it contains the date and time the ADIF file was written, which will change each time 'make unittest' is run.
-        f.close()
-
-        self.connection.close()
-
-    def test_adif_is_valid(self):
-        """ Check that ADIF field validation is working correctly for different data types. """
-        assert(self.adif.is_valid("CALL", "TEST123", "S"))
-        assert(self.adif.is_valid("QSO_DATE", "20120402", "D"))
-        assert(self.adif.is_valid("TIME_ON", "1230", "T"))
-        assert(self.adif.is_valid("TX_PWR", "5", "N"))
-
-if(__name__ == '__main__'):
-    unittest.main()
