@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-#    Copyright (C) 2013-2016 Christian T. Jacobs.
+#    Copyright (C) 2013-2017 Christian Thomas Jacobs.
 
 #    This file is part of PyQSO.
 
@@ -17,16 +17,19 @@
 #    You should have received a copy of the GNU General Public License
 #    along with PyQSO.  If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, GObject
+from gi.repository import GObject
 import logging
 from datetime import datetime
+from os.path import expanduser
+try:
+    import configparser
+except ImportError:
+    import ConfigParser as configparser
 try:
     import numpy
     logging.info("Using version %s of numpy." % (numpy.__version__))
     import matplotlib
     logging.info("Using version %s of matplotlib." % (matplotlib.__version__))
-    matplotlib.use('Agg')
-    matplotlib.rcParams['font.size'] = 10.0
     import mpl_toolkits.basemap
     logging.info("Using version %s of mpl_toolkits.basemap." % (mpl_toolkits.basemap.__version__))
     from matplotlib.backends.backend_gtk3cairo import FigureCanvasGTK3Cairo as FigureCanvas
@@ -37,26 +40,43 @@ except ImportError as e:
     have_necessary_modules = False
 
 
-class GreyLine(Gtk.VBox):
+class GreyLine:
 
     """ A tool for visualising the grey line. """
 
-    def __init__(self, parent):
+    def __init__(self, application):
         """ Set up the drawing canvas and the timer which will re-plot the grey line every 30 minutes.
 
-        :arg parent: The parent Gtk window.
+        :arg application: The PyQSO application containing the main Gtk window, etc.
         """
         logging.debug("Setting up the grey line...")
-        Gtk.VBox.__init__(self, spacing=2)
-        self.parent = parent
+
+        self.application = application
+        self.builder = self.application.builder
+
+        # Get the QTH coordinates, if available.
+        config = configparser.ConfigParser()
+        have_config = (config.read(expanduser('~/.config/pyqso/preferences.ini')) != [])
+        (section, option) = ("general", "show_qth")
+        self.show_qth = False
+        if(have_config and config.has_option(section, option)):
+            if(config.getboolean(section, option)):
+                self.show_qth = True
+                try:
+                    self.qth_name = config.get("general", "qth_name")
+                    self.qth_latitude = float(config.get("general", "qth_latitude"))
+                    self.qth_longitude = float(config.get("general", "qth_longitude"))
+                except ValueError:
+                    logging.warning("Unable to get the QTH name, latitude and/or longitude. The QTH will not be pinpointed on the grey line map. Check preferences?")
+                    self.show_qth = False
 
         if(have_necessary_modules):
             self.fig = matplotlib.figure.Figure()
             self.canvas = FigureCanvas(self.fig)  # For embedding in the Gtk application
-            self.pack_start(self.canvas, True, True, 0)
+            self.builder.get_object("greyline").pack_start(self.canvas, True, True, 0)
             self.refresh_event = GObject.timeout_add(1800000, self.draw)  # Re-draw the grey line automatically after 30 minutes (if the grey line tool is visible).
 
-        self.show_all()
+        self.builder.get_object("greyline").show_all()
 
         logging.debug("Grey line ready!")
 
@@ -70,7 +90,9 @@ class GreyLine(Gtk.VBox):
         """
 
         if(have_necessary_modules):
-            if(self.parent.toolbox.tools.get_current_page() != 1 or not self.parent.toolbox.get_visible()):
+            toolbox = self.builder.get_object("toolbox")
+            tools = self.builder.get_object("tools")
+            if(tools.get_current_page() != 1 or not toolbox.get_visible()):
                 # Don't re-draw if the grey line is not visible.
                 return True  # We need to return True in case this is method was called by a timer event.
             else:
@@ -81,15 +103,22 @@ class GreyLine(Gtk.VBox):
 
                 # Draw the map of the world. This is based on the example from:
                 # http://matplotlib.org/basemap/users/examples.html
-                m = mpl_toolkits.basemap.Basemap(projection='mill', lon_0=0, ax=sub, resolution='c', fix_aspect=False)
-                m.drawcountries(linewidth=0.5)
-                m.drawcoastlines(linewidth=0.5)
+                m = mpl_toolkits.basemap.Basemap(projection="mill", lon_0=0, ax=sub, resolution="c", fix_aspect=False)
+                m.drawcountries(linewidth=0.4)
+                m.drawcoastlines(linewidth=0.4)
                 m.drawparallels(numpy.arange(-90, 90, 30), labels=[1, 0, 0, 0])
                 m.drawmeridians(numpy.arange(m.lonmin, m.lonmax+30, 60), labels=[0, 0, 0, 1])
-                m.drawmapboundary(fill_color='lightblue')
-                m.fillcontinents(color='darkgreen', lake_color='lightblue')
+                m.drawmapboundary(fill_color="skyblue")
+                m.fillcontinents(color="green", lake_color="skyblue")
                 m.nightshade(datetime.utcnow())  # Add in the grey line using UTC time. Note that this requires NetCDF.
                 logging.debug("Grey line drawn.")
+
+                # Pin-point QTH on the map.
+                if(self.show_qth):
+                    qth_x, qth_y = m(self.qth_longitude, self.qth_latitude)
+                    m.plot(qth_x, qth_y, "ro")
+                    sub.text(qth_x+0.015*qth_x, qth_y+0.015*qth_y, self.qth_name, color="white", size="medium", weight="bold")
+
                 return True
         else:
             return False  # Don't try to re-draw the canvas if the necessary modules to do so could not be imported.
